#!/usr/bin/env python
# Copyright 2025 The OpenSSL Project Authors. All Rights Reserved.
#
# Licensed under the Apache License 2.0 (the "License").  You may not use
# this file except in compliance with the License.  You can obtain a copy
# in the file LICENSE in the source distribution or at
# https://www.openssl.org/source/license.html

# A python program written to parse (version 1) of the WYCHEPROOF test vectors for
# ML_DSA. The 6 files that can be processed by this utility can be downloaded
# from
#  https://github.com/C2SP/wycheproof/blob/8e7fa6f87e6993d7b613cf48b46512a32df8084a/testvectors_v1/mldsa_*_standard_*_test.json")
# and output from this utility to
# test/recipes/30-test_evp_data/evppkey_ml_dsa_44_wycheproof_sign.txt
# test/recipes/30-test_evp_data/evppkey_ml_dsa_65_wycheproof_sign.txt
# test/recipes/30-test_evp_data/evppkey_ml_dsa_87_wycheproof_sign.txt
# test/recipes/30-test_evp_data/evppkey_ml_dsa_44_wycheproof_verify.txt
# test/recipes/30-test_evp_data/evppkey_ml_dsa_65_wycheproof_verify.txt
# test/recipes/30-test_evp_data/evppkey_ml_dsa_87_wycheproof_verify.txt
#
# e.g. python3 ./test/mldsa_wycheproof_parse.py -alg ML-DSA-44 ./wycheproof/testvectors_v1/mldsa_44_standard_sign_test.json > test/recipes/30-test_evp_data/evppkey_ml_dsa_44_wycheproof_sign.txt

import json
import argparse
import datetime
from _ast import Or

def print_label(label, value):
    print(label + " = " + value)

def print_hexlabel(label, tag, value):
    print(label + " = hex" + tag + ":" + value)

def parse_ml_dsa_sig_gen(alg, groups):
    grpId = 1
    for grp in groups:
        keyOnly = False
        first = True
        name = alg.replace('-', '_')
        keyname = name + "_" + str(grpId)
        grpId += 1

        for tst in grp['tests']:
            if first:
                first = False
                if 'flags' in tst:
                    if 'IncorrectPrivateKeyLength' in tst['flags'] or 'InvalidPrivateKey' in tst['flags']:
                        keyOnly = True
                if not keyOnly:
                    print("")
                    print_label("PrivateKeyRaw", keyname + ":" + alg + ":" + grp['privateKey'])
            testname = name + "_" + str(tst['tcId'])
            print("\n# " + str(tst['tcId']) + " " + tst['comment'])

            print_label("FIPSversion", ">=3.5.0")
            if keyOnly:
                print_label("KeyFromData", alg)
                print_hexlabel("Ctrl", "priv", grp['privateKey'])
                print_label("Result", "KEY_FROMDATA_ERROR")
            else:
                print_label("Sign-Message", alg + ":" + keyname)
                print_label("Input", tst['msg'])
                print_label("Output", tst['sig'])
                if 'ctx' in tst:
                    print_hexlabel("Ctrl", "context-string", tst['ctx'])
                print_label("Ctrl", "message-encoding:1")
                print_label("Ctrl", "deterministic:1")
                if tst['result'] == "invalid":
                    print_label("Result", "PKEY_CTRL_ERROR")

def parse_ml_dsa_sig_ver(alg, groups):
    grpId = 1
    for grp in groups:
        keyOnly = False
        first = True
        name = alg.replace('-', '_')
        keyname = name + "_" + str(grpId)
        grpId += 1

        for tst in grp['tests']:
            if first:
                first = False
                if 'flags' in tst:
                    if 'IncorrectPublicKeyLength' in tst['flags'] or 'InvalidPublicKey' in tst['flags']:
                        keyOnly = True
                if not keyOnly:
                    print("")
                    print_label("PublicKeyRaw", keyname + ":" + alg + ":" + grp['publicKey'])
            testname = name + "_" + str(tst['tcId'])
            print("\n# " + str(tst['tcId']) + " " + tst['comment'])

            print_label("FIPSversion", ">=3.5.0")
            if keyOnly:
                print_label("KeyFromData", alg)
                print_hexlabel("Ctrl", "pub", grp['publicKey'])
                print_label("Result", "KEY_FROMDATA_ERROR")
            else:
                print_label("Verify-Message-Public", alg + ":" + keyname)
                print_label("Input", tst['msg'])
                print_label("Output", tst['sig'])
                if 'ctx' in tst:
                    print_hexlabel("Ctrl", "context-string", tst['ctx'])
                print_label("Ctrl", "message-encoding:1")
                print_label("Ctrl", "deterministic:1")
                if tst['result'] == "invalid":
                    if 'InvalidContext' in tst['flags']:
                        print_label("Result", "PKEY_CTRL_ERROR")
                    else:
                        print_label("Result", "VERIFY_ERROR")

parser = argparse.ArgumentParser(description="")
parser.add_argument('filename', type=str)
parser.add_argument('-alg', type=str)
args = parser.parse_args()

# Open and read the JSON file
with open(args.filename, 'r') as file:
    data = json.load(file)

year = datetime.date.today().year
version = data['generatorVersion']
algorithm = data['algorithm']
mode = data['testGroups'][0]['type']

print("# Copyright " + str(year) + " The OpenSSL Project Authors. All Rights Reserved.")
print("#")
print("# Licensed under the Apache License 2.0 (the \"License\").  You may not use")
print("# this file except in compliance with the License.  You can obtain a copy")
print("# in the file LICENSE in the source distribution or at")
print("# https://www.openssl.org/source/license.html\n")
print("# Wycheproof test data for " + algorithm + " " + mode + " generated from")
print("# https://github.com/C2SP/wycheproof/blob/8e7fa6f87e6993d7b613cf48b46512a32df8084a/testvectors_v1/mldsa_*_standard_*_test.json")

print("# [version " + str(version) + "]")

if algorithm == "ML-DSA":
    if mode == 'MlDsaSign':
        parse_ml_dsa_sig_gen(args.alg, data['testGroups'])
    elif mode == 'MlDsaVerify':
        parse_ml_dsa_sig_ver(args.alg, data['testGroups'])
    else:
        print("Unsupported mode " + mode)
else:
    print("Unsupported algorithm " + algorithm)



#!/usr/bin/env python3

# Copyright 2026 The OpenSSL Project Authors. All Rights Reserved.
#
# Licensed under the Apache License 2.0 (the "License").  You may not use
# this file except in compliance with the License.  You can obtain a copy
# in the file LICENSE in the source distribution or at
# https://www.openssl.org/source/license.html

# This script generates missing-kdf.der - a password-encrypted CMS message
# without the keyDerivationAlgorithm field, which is used in the
# “PWRI missing keyDerivationAlgorithm regression” test.
#
# Usage: python3 make_missing_kdf_der.py valid.der missing-kdf.der

from __future__ import annotations

import argparse
import sys
from dataclasses import dataclass
from pathlib import Path


@dataclass
class Node:
    off: int
    tag: int
    hdr_len: int
    length: int
    end: int
    children: list["Node"]


def read_len(data: bytes, off: int) -> tuple[int, int]:
    first = data[off]
    if first < 0x80:
        return first, 1
    n = first & 0x7F
    if n == 0 or n > 4:
        raise ValueError(f"unsupported DER length form at {off}")
    val = 0
    for b in data[off + 1 : off + 1 + n]:
        val = (val << 8) | b
    return val, 1 + n


def parse_node(data: bytes, off: int) -> Node:
    tag = data[off]
    length, len_len = read_len(data, off + 1)
    hdr_len = 1 + len_len
    end = off + hdr_len + length
    children: list[Node] = []
    if tag & 0x20:
        cur = off + hdr_len
        while cur < end:
            child = parse_node(data, cur)
            children.append(child)
            cur = child.end
        if cur != end:
            raise ValueError(f"child parse ended at {cur}, expected {end}")
    return Node(off=off, tag=tag, hdr_len=hdr_len, length=length, end=end, children=children)


def encode_len(length: int, existing_len_len: int) -> bytes:
    if existing_len_len == 1:
        if length >= 0x80:
            raise ValueError("new length no longer fits in short-form DER")
        return bytes([length])
    payload_len = existing_len_len - 1
    max_len = (1 << (payload_len * 8)) - 1
    if length > max_len:
        raise ValueError("new length no longer fits in existing long-form DER")
    out = bytearray([0x80 | payload_len])
    for shift in range((payload_len - 1) * 8, -8, -8):
        out.append((length >> shift) & 0xFF)
    return bytes(out)


def patch_length_field(buf: bytearray, node: Node, delta: int) -> None:
    new_len = node.length + delta
    if new_len < 0:
        raise ValueError("negative patched length")
    len_bytes = encode_len(new_len, node.hdr_len - 1)
    start = node.off + 1
    end = start + len(node.hdr_len.to_bytes(1, "big")) - 1  # unused, kept for clarity
    buf[start : start + len(len_bytes)] = len_bytes


def main() -> int:
    ap = argparse.ArgumentParser(description="Remove PWRI keyDerivationAlgorithm from a CMS DER blob.")
    ap.add_argument("input_der")
    ap.add_argument("output_der")
    args = ap.parse_args()

    data = Path(args.input_der).read_bytes()
    root = parse_node(data, 0)

    # CMS structure we expect:
    # SEQUENCE { OID envelopedData, [0] SEQUENCE { version, SET recipientInfos, ... } }
    ed_wrapper = root.children[1]
    env_seq = ed_wrapper.children[0]
    recipient_set = env_seq.children[1]
    pwri_choice = recipient_set.children[0]  # [3]

    if pwri_choice.tag != 0xA3:
        raise ValueError(f"expected PWRI choice tag 0xA3, found 0x{pwri_choice.tag:02x}")
    if len(pwri_choice.children) < 3:
        raise ValueError("unexpected PWRI child count")

    version = pwri_choice.children[0]
    maybe_kdf = pwri_choice.children[1]
    keyenc = pwri_choice.children[2]
    if version.tag != 0x02:
        raise ValueError("PWRI version is not INTEGER")
    if maybe_kdf.tag != 0xA0:
        raise ValueError(f"PWRI child after version is not [0] keyDerivationAlgorithm: 0x{maybe_kdf.tag:02x}")
    if keyenc.tag != 0x30:
        raise ValueError("PWRI keyEncryptionAlgorithm is not SEQUENCE")

    remove_start = maybe_kdf.off
    remove_end = maybe_kdf.end
    remove_len = remove_end - remove_start

    out = bytearray(data)
    del out[remove_start:remove_end]

    # Adjust ancestors whose length spans the removed field.
    for node in [root, ed_wrapper, env_seq, recipient_set, pwri_choice]:
        patch_length_field(out, node, -remove_len)

    Path(args.output_der).write_bytes(out)
    print(f"removed {remove_len} bytes at [{remove_start}, {remove_end})")
    return 0


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
# Copyright 2025 The OpenSSL Project Authors. All Rights Reserved.
#
# Licensed under the Apache License 2.0 (the "License").  You may not use
# this file except in compliance with the License.  You can obtain a copy
# in the file LICENSE in the source distribution or at
# https://www.openssl.org/source/license.html

# A python program written to parse (version 1.0) of the ACVP test vectors for
# LMS. The file that can be processed by this utility can be downloaded from
#  https://raw.githubusercontent.com/usnistgov/ACVP-Server/refs/heads/master/gen-val/json-files/LMS-sigVer-1.0/internalProjection.json
# and output from this utility to
#  test/recipes/30-test_evp_data/evppkey_lms.txt
#
# e.g. python3 mldsa_parse.py ~/Downloads/internalProjection.json > ./test/recipes/30-test_evp_data/evppkey_lms.txt
#
import json
import argparse
import datetime

def print_label(label, value):
    print(label + " = " + value)

def print_hexlabel(label, tag, value):
    print(label + " = hex" + tag + ":" + value)

def parse_lms_sig_ver(groups):
    for grp in groups:
        lmsmode = grp["lmsMode"]
        lmotsmode = grp["lmOtsMode"]
        name = lmsmode + "_" + str(grp["tgId"])
        pubkey = grp["publicKey"]

        if grp["testType"] != "AFT":
            continue

        print_label("Title", lmsmode + " " + lmotsmode)
        print("");
        print_label("PublicKeyRaw", name + ":" + "LMS" + ":" + pubkey)
        for tst in grp['tests']:
            testname = lmsmode + "_" + str(tst['tcId'])
            print("");
            if "reason" in tst:
                print("# " + tst['reason'])
            print_label("FIPSversion", ">=3.6.0")
            print_label("Verify-Message-Public", "LMS:" + name)
            print_label("Input", tst['message'])
            print_label("Output", tst['signature'])
            if not tst['testPassed']:
                print_label("Result", "VERIFY_ERROR")

parser = argparse.ArgumentParser(description="")
parser.add_argument('filename', type=str)
args = parser.parse_args()

# Open and read the JSON file
with open(args.filename, 'r') as file:
    data = json.load(file)

year = datetime.date.today().year
version = data['vsId']
revision = data['revision']
algorithm = data['algorithm']

print("# Copyright " + str(year) + " The OpenSSL Project Authors. All Rights Reserved.")
print("#")
print("# Licensed under the Apache License 2.0 (the \"License\").  You may not use")
print("# this file except in compliance with the License.  You can obtain a copy")
print("# in the file LICENSE in the source distribution or at")
print("# https://www.openssl.org/source/license.html\n")
print("# ACVP test data for " + algorithm + " generated from")
print("# https://raw.githubusercontent.com/usnistgov/ACVP-Server/refs/heads/master/gen-val/json-files/LMS-sigVer-1.0/internalProjection.json")
print("# [version " + str(version) + " : revision " + str(revision) + "]")
print("")

if algorithm == "LMS":
    parse_lms_sig_ver(data['testGroups'])
else:
    print("Unsupported algorithm " + algorithm)



#!/usr/bin/env python
# Copyright 2025 The OpenSSL Project Authors. All Rights Reserved.
#
# Licensed under the Apache License 2.0 (the "License").  You may not use
# this file except in compliance with the License.  You can obtain a copy
# in the file LICENSE in the source distribution or at
# https://www.openssl.org/source/license.html

# A python program written to parse (version 42) of the ACVP test vectors for
# ML_DSA. The 3 files that can be processed by this utility can be downloaded
# from
#  https://github.com/usnistgov/ACVP-Server/blob/master/gen-val/json-files/ML-DSA-keyGen-FIPS204/internalProjection.json
#  https://github.com/usnistgov/ACVP-Server/blob/master/gen-val/json-files/ML-DSA-sigGen-FIPS204/internalProjection.json
#  https://github.com/usnistgov/ACVP-Server/blob/master/gen-val/json-files/ML-DSA-sigVer-FIPS204/internalProjection.json
# and output from this utility to
#  test/recipes/30-test_evp_data/evppkey_ml_dsa_keygen.txt
#  test/recipes/30-test_evp_data/evppkey_ml_dsa_siggen.txt
#  test/recipes/30-test_evp_data/evppkey_ml_dsa_sigver.txt
#
# e.g. python3 mldsa_parse.py ~/Downloads/keygen.json > ./test/recipes/30-test_evp_data/evppkey_ml_dsa_keygen.txt
#
import json
import argparse
import datetime

def print_label(label, value):
    print(label + " = " + value)

def print_hexlabel(label, tag, value):
    print(label + " = hex" + tag + ":" + value)

def parse_ml_dsa_key_gen(groups):
    for grp in groups:
        for tst in grp['tests']:
            print("");
            print_label("FIPSversion", ">=3.5.0")
            print_label("KeyGen", grp['parameterSet'])
            print_label("KeyName", "tcId" + str(tst['tcId']))
            print_hexlabel("Ctrl", "seed", tst['seed'])
            print_hexlabel("CtrlOut", "pub", tst['pk'])
            print_hexlabel("CtrlOut", "priv", tst['sk'])

def parse_ml_dsa_sig_gen(groups):
    for grp in groups:
        deter = grp['deterministic'] # Boolean
        externalMu = grp['externalMu'] # Boolean
        signInterfaceExternal = (grp['signatureInterface'] == "External")
        signPreHash = (grp['preHash'] == "preHash")
        signPure = (grp['preHash'] == "pure")
        includeMu = True # Flag flips to only include the Ctrl mu:0 half the time

        if signPreHash:
            continue
        if not externalMu and not signPure:
            continue

        name = grp['parameterSet'].replace('-', '_')
        for tst in grp['tests']:
            testname = name + "_" + str(tst['tcId'])
            print("");
            print_label("PrivateKeyRaw", testname + ":" + grp['parameterSet'] + ":" + tst['sk'])
            print("");
            print_label("FIPSversion", ">=3.5.0")
            print_label("Sign-Message", grp['parameterSet'] + ":" + testname)
            print_label("Input", tst['mu' if externalMu else 'message'])
            print_label("Output", tst['signature'])
            print_label("Ctrl", "message-encoding:1")
            if not externalMu:
                print_label("Ctrl", "hexcontext-string:" + tst["context"])
                includeMu = not includeMu
            if externalMu or includeMu:
                print_label("Ctrl", "mu:" + ("1" if externalMu else "0"))
            print_label("Ctrl", "deterministic:" + ("1" if deter else "0"))
            if not deter:
                print_label("Ctrl", "hextest-entropy:" + tst["rnd"])

def parse_ml_dsa_sig_ver(groups):
    for grp in groups:
        externalMu = grp["externalMu"] # Boolean
        signInterfaceExternal = (grp['signatureInterface'] == "External")
        signPreHash = (grp['preHash'] == "preHash")
        signPure = (grp['preHash'] == "pure")
        includeMu = True # Flag flips to only include the Ctrl mu:0 half the time

        if signPreHash:
            continue
        if not externalMu and not signPure:
            continue

        name = grp['parameterSet'].replace('-', '_')
        for tst in grp['tests']:
            testname = name + "_" + str(tst['tcId'])
            print("");
            print_label("PublicKeyRaw", testname + ":" + grp['parameterSet'] + ":" + tst['pk'])
            print("");
            if "reason" in tst:
                print("# " + tst['reason'])
            print_label("FIPSversion", ">=3.5.0")
            print_label("Verify-Message-Public", grp['parameterSet'] + ":" + testname)
            print_label("Input", tst['mu' if externalMu else 'message'])
            print_label("Output", tst['signature'])
            print_label("Ctrl", "message-encoding:1")
            if not externalMu:
                print_label("Ctrl", "hexcontext-string:" + tst["context"])
                includeMu = not includeMu
            if externalMu or includeMu:
                print_label("Ctrl", "mu:" + ("1" if externalMu else "0"))
            if not tst['testPassed']:
                print_label("Result", "VERIFY_ERROR")

parser = argparse.ArgumentParser(description="")
parser.add_argument('filename', type=str)
args = parser.parse_args()

# Open and read the JSON file
with open(args.filename, 'r') as file:
    data = json.load(file)

year = datetime.date.today().year
version = data['vsId']
algorithm = data['algorithm']
mode = data['mode']

print("# Copyright " + str(year) + " The OpenSSL Project Authors. All Rights Reserved.")
print("#")
print("# Licensed under the Apache License 2.0 (the \"License\").  You may not use")
print("# this file except in compliance with the License.  You can obtain a copy")
print("# in the file LICENSE in the source distribution or at")
print("# https://www.openssl.org/source/license.html\n")
print("# ACVP test data for " + algorithm + " " + mode + " generated from")
print("# https://github.com/usnistgov/ACVP-Server/blob/master/gen-val/json-files/"
      "ML-DSA-" + mode + "-FIPS204/internalProjection.json")
print("# [version " + str(version) + "]")

if algorithm == "ML-DSA":
    if mode == 'sigVer':
        parse_ml_dsa_sig_ver(data['testGroups'])
    elif mode == 'sigGen':
        parse_ml_dsa_sig_gen(data['testGroups'])
    elif mode == 'keyGen':
        parse_ml_dsa_key_gen(data['testGroups'])
    else:
        print("Unsupported mode " + mode)
else:
    print("Unsupported algorithm " + algorithm)
